// BDE - (C) Copyright 1995 by Borland International

// Filter.c
#include "snipit.h"

static const char szTblName[] = "cust";
static const char szTblType[] = szDBASE;

//=====================================================================
//  Function:
//          Filter();
//
//  Description:
//          This example shows how to use filters to limit the result
//          set of a table. Filters perform a function similar to that
//          of ranges, but more powerful operations are supported.
//=====================================================================
void
Filter (void)
{
    hDBIDb          hDb;            // Handle to the database.
    hDBICur         hCur;           // Handle to the table.
    pBYTE           pcanExpr;       // Structure containing filter info.
    hDBIFilter      hFilter;        // Filter handle.
    UINT16          uSizeNodes;     // Size of the nodes in the tree.
    UINT16          uSizeCanExpr;   // Size of the header information.
    UINT16          uSizeLiterals;  // Size of the literals.
    UINT16          uTotalSize;     // Total size of the filter expression.
    UINT32          uNumRecs  = 10; // Number of records to display.
    CANExpr         canExp;         // Contains the header information.
    UINT16          Nodes[] =       // Nodes of the filter tree.
            {
                // Offset 0
                nodeBINARY,     // canBinary.nodeClass
                canGT,          // canBinary.canOp
                8,              // canBinary.iOperand1
                16,             // canBinary.iOperand2
                                //   Offsets in the Nodes array

                // Offset 8
                nodeFIELD,      // canField.nodeClass
                canFIELD,       // canField.canOp
                1,              // canField.iFieldNum
                0,              // canField.iNameOffset: szField is the
                                //   literal at offset 0

                // Offset 16
                nodeCONST,      // canConst.nodeClass
                canCONST,       // canConst.canOp
                fldFLOAT,       // canConst.iType
                8,              // canConst.iSize
                8               // canConst.iOffset: fconst is the
                                //   literal at offset strlen(szField) + 1
            };

    CHAR            szField[] = "CUST_NO";  // Name of the field for the
                                            //   third node of the tree.
    DFLOAT          fConst    = 1500.0;     // Value of the constant for
                                            //   the second node of the tree.
    DBIResult       rslt;                   // Return value from IDAPI
                                            //   functions.

    Screen("*** Filter Example ***\r\n");

    BREAK_IN_DEBUGGER();

    Screen("    Initializing IDAPI...");
    if (InitAndConnect(&hDb) != DBIERR_NONE)
    {                                        
        Screen("\r\n*** End of Example ***");
        return;
    }

    Screen("    Setting the database directory...");
    rslt = DbiSetDirectory(hDb, (pCHAR) szTblDirectory);
    ChkRslt(rslt, "SetDirectory");

    Screen("    Open the %s table...", szTblName);
    rslt = DbiOpenTable(hDb, (pCHAR) szTblName, (pCHAR) szTblType,
                        NULL, NULL, 0, dbiREADWRITE, dbiOPENSHARED,
                        xltFIELD, FALSE, NULL, &hCur);
    if (ChkRslt(rslt, "OpenTable") != DBIERR_NONE)
    {
        CloseDbAndExit(&hDb);
        Screen("\r\n*** End of Example ***");
        return;
    }

    // Go to the beginning of the table
    rslt = DbiSetToBegin(hCur);
    ChkRslt(rslt, "SetToBegin");

    Screen("\r\n    Display the %s table...", szTblName);
    DisplayTable(hCur, uNumRecs);

    uSizeNodes      = sizeof(Nodes); // Size of the nodes.
    uSizeLiterals  = strlen(szField) + 1 + sizeof(fConst); // Size of the
                                                           //  literals.
    uSizeCanExpr    = sizeof(CANExpr); // Size of the header information.
    uTotalSize      = uSizeCanExpr + uSizeNodes + uSizeLiterals; // Total size
                                                               // of the filter.

    // Initialize the header information
    canExp.iVer = 1; // Version.
    canExp.iTotalSize = uTotalSize; // Total size of the filter.
    canExp.iNodes = 3; // Number of nodes.
    canExp.iNodeStart = uSizeCanExpr; // The offset in the buffer where the
                                      //   expression nodes start.
    canExp.iLiteralStart = uSizeCanExpr + uSizeNodes; // The offset in the
                                                      //   buffer where the
                                                      //   literals start.
    
    // Allocate space for the filter expression. 
    pcanExpr = (pBYTE)malloc(uTotalSize * sizeof(BYTE));
    if (pcanExpr == NULL)
    {
        Screen("    Could not allocate memory...");
        DbiCloseCursor(&hCur);
        CloseDbAndExit(&hDb);
        Screen("\r\n*** End of Example ***");
        return;
    }
    
    // Initialize the filter expression.
    memmove(pcanExpr, &canExp, uSizeCanExpr);
    memmove(&pcanExpr[uSizeCanExpr], Nodes, uSizeNodes);

    memmove(&pcanExpr[uSizeCanExpr + uSizeNodes],
            szField, strlen(szField) + 1); // First litteral
    memmove(&pcanExpr[uSizeCanExpr + uSizeNodes + strlen(szField) + 1],
            &fConst, sizeof(fConst)); // Second litteral

    rslt = DbiSetToBegin(hCur);
    ChkRslt(rslt, "SetToBegin");

    Screen("\r\n    Add a filter to the %s table which will limit"
           " the records\r\n        which are displayed to those whose"
           " %s field is greater than %.1lf...", szTblName, szField, fConst);
    rslt = DbiAddFilter(hCur, 0L, 1, FALSE, (pCANExpr)pcanExpr, NULL,
                        &hFilter);
    if (ChkRslt(rslt, "AddFilter") != DBIERR_NONE)
    {
        rslt = DbiCloseCursor(&hCur);
        ChkRslt(rslt, "CloseCursor");
        free(pcanExpr);
        CloseDbAndExit(&hDb);
        Screen("\r\n*** End of Example ***");
        return;
    }

    // Activate the filter.
    Screen("    Activate the filter on the %s table...", szTblName);
    rslt = DbiActivateFilter(hCur, hFilter);
    ChkRslt(rslt, "ActivateFilter");

    rslt = DbiSetToBegin(hCur);
    ChkRslt(rslt, "SetToBegin");

    Screen("\r\n    Display the %s table with the filter set...", szTblName);
    DisplayTable(hCur, uNumRecs);

    Screen("\r\n    Deactivate the filter...");
    rslt = DbiDeactivateFilter(hCur, hFilter);
    ChkRslt(rslt, "DeactivateFilter");

    Screen("\r\n    Drop the filter...");
    rslt = DbiDropFilter(hCur, hFilter);
    ChkRslt(rslt, "DropFilter");

    rslt = DbiCloseCursor(&hCur);
    ChkRslt(rslt, "CloseCursor");

    free(pcanExpr);

    Screen("    Close the database and exit IDAPI...");
    CloseDbAndExit(&hDb);

    Screen("\r\n*** End of Example ***");
}
